import { ReactElement, useRef } from 'react';
import { createPortal } from 'react-dom';
import { useMount, useUnmount } from '@gilbarbara/hooks';

import { canUseDOM } from '~/modules/dom';

interface Props {
  children: ReactElement;
  id: string;
}

export default function JoyridePortal(props: Props) {
  const { children, id } = props;
  const nodeRef = useRef<HTMLElement | null>(null);

  useMount(() => {
    if (!canUseDOM()) {
      return;
    }

    nodeRef.current = document.createElement('div');
    nodeRef.current.id = id;

    document.body.appendChild(nodeRef.current);
  });

  useUnmount(() => {
    if (!canUseDOM() || !nodeRef.current) {
      return;
    }

    if (nodeRef.current.parentNode === document.body) {
      document.body.removeChild(nodeRef.current);
      nodeRef.current = null;
    }
  });

  if (!canUseDOM() || !nodeRef.current) {
    return null;
  }

  return createPortal(children, nodeRef.current);
}
